/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

// SPDX-FileCopyrightText: 2021 - 2025 Kohei Yoshida
//
// SPDX-License-Identifier: MIT

#pragma once

#include "./multi_type_vector/aos/iterator.hpp"

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
